/*
Copyright  2006, Bilal Haidar
All rights reserved.
http://www.bhaidar.net

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

- Redistributions of source code _MembershipUserst retain the above copyright
notice, this list of conditions and the following disclaimer.

- Neither Bilal Haidar, nor the names of its
contributors may be used to endorse or promote products
derived from this software without specific prior written 
permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Data;
using System.Configuration;
using System.Web.Configuration;
using System.Configuration.Provider;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;    
using System.Globalization;
using System.Collections.ObjectModel;
using System.Web;
using System.Web.Hosting;
using System.Web.Security;
using System.Security.Principal;
using System.Web.UI;
using System.Text;
using System.Threading;


namespace ExtendedMemberShip
{
    /// <summary>
    /// MemberShip class acting upon default ASP.NET 2.0 Membership API
    /// and the custom membership provider created
    /// </summary>
    public class MemberShip
    {
        ///// <summary>
        ///// This is the simplest method to create a new user.
        ///// </summary>
        ///// <param name="UserName">Username of the user</param>
        ///// <param name="Password">Password of the user</param>
        ///// <returns>MemberShipUser instance of the created user</returns>
        //public static MemberShipUser CreateUser(string UserName, string Password, out MembershipCreateStatus status)
        //{
        //    // Call the semi-feature CreateUser method
        //    MemberShipUser msu = MemberShip.CreateUser(UserName, Password, null, null, DateTime.MinValue, out status);

        //    if (status != MembershipCreateStatus.Success)
        //    {
        //        return null;
        //    }

        //    return msu;
        //}

        /// <summary>
        /// This method takes the custom parameters to create the user
        /// in our custom table. I had to add this method to be able to use it
        /// with the CreateUserWizard control.
        /// </summary>
        /// <param name="UserName">Username of the user</param>
        /// <param name="FirstName">First name of the user</param>
        /// <param name="LastName">Last name of the user</param>
        /// <param name="DateOfBirth">Date of birth of the user</param>
        /// <returns>True/False</returns>
        public static bool CreateUser(string UserName, string FirstName, string LastName, DateTime DateOfBirth)
        {
            // Get the UserId
            object UserId = Membership.GetUser(UserName).ProviderUserKey;

            // Call the custom provider
            return MemberShip.Provider.CreateUser(new UserInfo(UserId, FirstName, LastName, DateOfBirth));
        }
        
        /// <summary>
        /// This is full-feature modified CreateUser method that adds a new user to the
        /// the default Membership API database and our custom database.
        /// </summary>
        /// <param name="UserName">Username of the user</param>
        /// <param name="Password">Password of the user</param>
        /// <param name="Email">Email of the user</param>
        /// <param name="PasswordQuestion">Password question used by the user to retreive later on his/her password</param>
        /// <param name="PasswordAnswer">This is the answer of the above question</param>
        /// <param name="IsApproved">Whether the user is approved or not</param>
        /// <param name="ProviderUserKey">The internal UserId in the database</param>
        /// <param name="FirstName">First name of the user</param>
        /// <param name="LastName">Last name of the user</param>
        /// <param name="DateOfBirth">Date of birth of the user</param>
        /// <param name="status">Status of the created user</param>
        /// <returns>An instance of the custom object MemberShipUser</returns>
        public static MemberShipUser CreateUser(string UserName, string Password, string Email, string PasswordQuestion,
            string PasswordAnswer, bool IsApproved, object ProviderUserKey, string FirstName, string LastName, 
            DateTime DateOfBirth, out MembershipCreateStatus status)
        {
            // Call the default Membership CreateUser method
            MembershipUser _mu = Membership.CreateUser(UserName, Password, Email, PasswordQuestion, PasswordAnswer, IsApproved, out status);

            if (status != MembershipCreateStatus.Success)
            {
                return null;
            }

            // Now since the CreateUser was successful, we will add the additional
            // data to our custom table
            UserInfo _ui = new UserInfo(_mu.ProviderUserKey, FirstName, LastName, DateOfBirth);
            if (MemberShip.Provider.CreateUser(_ui) == true)
                return new MemberShipUser(_mu, FirstName, LastName, DateOfBirth);

            // Since the record was not created in our custom table
            // we need to remove it also from the default membership database
            MemberShip.DeleteUser(UserName);

            return null;
        }

        /// <summary>
        /// This method is used to delete a user
        /// and our custom table.
        /// </summary>
        /// <param name="UserName">Username of the user</param>
        /// <returns>A boolean whether the user was deleted or not</returns>
        public static bool DeleteUser(string UserName)
        {
            // A simple call to the default DeleteUser method
            // specifying that all related data are to be deleted

            // Get the userid to delete
            object UserId = Membership.GetUser(UserName).ProviderUserKey;

            if (UserId != null)
            {
                if (Membership.DeleteUser(UserName, true) == true)
                {
                    // We can delete from our custom table
                    // Execute our custom method to delete from our custom table
                    return MemberShip.Provider.Delete(UserId);
                }
            }

            return false;
        }

        /// <summary>
        /// This is the default update method updated to be used against
        /// the default and custom data
        /// </summary>
        /// <param name="_MemberShipUser">Takes as input a MemberShipUser instance</param>
        static public void Update(MemberShipUser msu)
        {
            // Flags used to check whether it is necessary to update the user
            // if no changes are there or just update the user
            // just to preserve a database call
            bool IsDefaultDirty = false;
            bool IsCustomDirty = false;

            // Get the user using our local GetUser method
            MemberShipUser _msu = MemberShip.GetUser(msu.UserName);

            // Check default properties
            if (_msu.Comment == null || _msu.Comment.CompareTo(msu.Comment) != 0)
            {
                IsDefaultDirty = true;
                _msu.Comment = msu.Comment;
            }

            if (_msu.Email == null || _msu.Email.CompareTo(msu.Email) != 0)
            {
                IsDefaultDirty = true;
                _msu.Email = msu.Email;
            }

            if (_msu.IsApproved != msu.IsApproved)
            {
                IsDefaultDirty = true;
                _msu.IsApproved = msu.IsApproved;
            }

            if (_msu.DateOfBirth != msu.DateOfBirth)
            {
                IsCustomDirty = true;
                _msu.DateOfBirth = msu.DateOfBirth;
            }

            // If there are any changes in the default MembershipUser, update it
            MembershipUser _mu = null;
            if (IsDefaultDirty == true)
            {
                // Get the default MembershipUser from the MemberShipUser object
                _mu = new MembershipUser(_msu.ProviderName, _msu.UserName, _msu.ProviderUserKey, _msu.Email, _msu.PasswordQuestion,
                    _msu.Comment, _msu.IsApproved, _msu.IsLockedOut, _msu.CreationDate, _msu.LastLoginDate, _msu.LastActivityDate,
                    _msu.LastPasswordChangedDate, _msu.LastLockoutDate);

                // Call default Membership Update method
                Membership.UpdateUser(_mu);
            }

            // check if the custom data needs to be updated
            if (IsCustomDirty == true)
            {
                // Update our custom data
                MemberShip.Provider.UpdateUser(_mu.ProviderUserKey,_msu.DateOfBirth);
                
            }
        }

        /// <summary>
        /// This is the simplest form of method GetUser
        /// It will return the currently logged in user
        /// </summary>
        /// <returns>A MemberShipUser instance</returns>
        public static MemberShipUser GetUser()
        {
            // Call our full featured GetUser method
            return MemberShip.GetUser(MemberShip.GetCurrentUserName(), true);
        }

        /// <summary>
        /// This method takes as input a UserName, and returns the user object
        /// who is not logged in.
        /// </summary>
        /// <returns>A MemberShipUser instance</returns>
        public static MemberShipUser GetUser(string UserName)
        {
            // Call our full featured GetUser method
            return MemberShip.GetUser(UserName, false);
        }

        /// <summary>
        /// This is one of the two full-featurede methods used to retreive a MemberShipUser object
        /// stored in the database
        /// </summary>
        /// <param name="UserName">Username of the user to be retreived</param>
        /// <param name="UserIsOnline">Specifies whether the user is online or not.</param>
        public static MemberShipUser GetUser(string UserName, bool UserIsOnline)
        {
            // Get the user from the default membership
            MembershipUser _mu = Membership.GetUser(UserName,UserIsOnline);

            // Check that the returned user is not null
            if (_mu != null)
            {
                // Get the UserId 
                object UserId = _mu.ProviderUserKey;
    
                // Get the additional related data from our custom database
                UserInfo _ui = MemberShip.Provider.GetUser(UserId);

                if (_ui != null)
                {
                    // Combine both objects to get a full one
                    return new MemberShipUser(_mu, _ui.FirstName, _ui.LastName, _ui.DateOfBirth);
                }

                // The returned user object from our custom database is null
                return null;
            }

            // The returned user object from the membership database is null
            return null;
        }

        /// <summary>
        /// This is the default method to get all users in the membership and our custom database
        /// </summary>
        public static List<MemberShipUser> GetAllUsers()
        {
            // calls our method to method to get all users from both
            // membership and our custom database
            int num1 = 0;
            return MemberShip.GetAllUsers(0, 0x7fffffff, out num1);
        }

        /// <summary>
        /// This is the full-featured method to get all users from databsae
        /// including our custom database and the membership database
        /// </summary>
        /// <param name="pageIndex">The location where to start</param>
        /// <param name="pageSize">Number of records to return</param>
        /// <param name="totalRecords">Total number of records in the db</param>
        /// <returns></returns>
        public static List<MemberShipUser> GetAllUsers(int pageIndex, int pageSize, out int totalRecords)
        {
            // Get all users from membership database
            MembershipUserCollection _muc = Membership.GetAllUsers(pageIndex, pageSize, out totalRecords);

            if (_muc != null)
            {
                // Single MemberShipUser instance
                MemberShipUser _msu = new MemberShipUser();

                // StringBuilder to hold all user ids
                StringBuilder sbUserIds = new StringBuilder();

                // Convert the MembershipCollection to a list
                List<MembershipUser> _muList = new List<MembershipUser>();
                foreach (MembershipUser _mu1 in _muc)
                {
                    _muList.Add(_mu1);
                    sbUserIds.AppendFormat("'{0}',", _mu1.ProviderUserKey);
                }

                // Get all users from both membership and custom database
                List<MemberShipUser> _msuList = new List<MemberShipUser>();

                // If there are element
                if (sbUserIds.Length > 0)
                {
                    // Calling method GetAllUsers with a list of user ids as an input
                    List<UserInfo> _uiList = new List<UserInfo>();
                    _uiList = MemberShip.Provider.GetUsersByUserIds(sbUserIds.ToString().Remove(sbUserIds.Length - 1));

                    if ((_uiList != null) && (_uiList.Count == _muc.Count))
                    {
                        // Loop though both lists to join them
                        int counter = _muc.Count;
                        // MembershipUser List
                        for (int i = 0; i < counter; i++)
                        {
                            // UserInfo List
                            for(int j=0; j<counter;j++)
                            {
                                if (_uiList[j].UserId.ToString().CompareTo(_muList[i].ProviderUserKey.ToString()) == 0)
                                    _msuList.Add(new MemberShipUser(_muList[i], _uiList[j].FirstName, _uiList[j].LastName, _uiList[j].DateOfBirth));
                            }
                        }

                        if (_msuList.Count > 0)
                            return _msuList;
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// Helper method used to get the username of the currently logged in user
        /// It can be accessed anywhere in the application
        /// </summary>
        /// <returns></returns>
        public static string GetCurrentUserName()
        {
            if (HostingEnvironment.IsHosted)
            {
                HttpContext context1 = HttpContext.Current;
                if (context1 != null)
                {
                    return context1.User.Identity.Name;
                }
            }
            IPrincipal principal1 = Thread.CurrentPrincipal;
            if ((principal1 != null) && (principal1.Identity != null))
            {
                return principal1.Identity.Name;
            }
            return string.Empty;
        }

        #region Provider Configuration
        // Initialization related variables and logic
		private static bool isInitialized = false;
		private static Exception initializationException;

		private static object initializationLock = new object();

		static MemberShip()
		{
			Initialize();
		}

		private static void Initialize()
		{
			try
			{
				// Get the feature's configuration info
				MemberShipConfiguration qc = (MemberShipConfiguration)ConfigurationManager.GetSection("MemberShip");

				if (qc.DefaultProvider == null || qc.Providers == null || qc.Providers.Count < 1)
					throw new ProviderException("You must specify a valid default provider.");

				// Instantiate the providers
				providerCollection = new MemberShipProviderCollection();
				ProvidersHelper.InstantiateProviders(qc.Providers, providerCollection, typeof(MemberShipProvider));
				providerCollection.SetReadOnly();
				defaultProvider = providerCollection[qc.DefaultProvider];
				if (defaultProvider == null)
				{
					throw new ConfigurationErrorsException("You must specify a default provider for the feature.",
						qc.ElementInformation.Properties["defaultProvider"].Source,
						qc.ElementInformation.Properties["defaultProvider"].LineNumber);
				}
			}
			catch (Exception ex)
			{
				initializationException = ex;
				isInitialized = true;
				throw ex;
			}

			isInitialized = true; //error-free initialization
		}

		// Public feature API
		private static MemberShipProvider defaultProvider;
		private static MemberShipProviderCollection providerCollection;

		public static MemberShipProvider Provider
		{
			get
			{
					return defaultProvider;
			}
		}

		public static MemberShipProviderCollection Providers
		{
			get
			{
					return providerCollection;
			}
		}
        #endregion
    }
}
/*
       /// <summary>
        /// Return a collection of MembershipUserWrapper's based on criteria passed in as parameters
        /// </summary>
        /// <param name="returnAllApprovedUsers">returns all users with approval set to true</param>
        /// <param name="returnAllNotApproviedUsers">return all users with approval set to false</param>
        /// <param name="usernameToFind">return based on username (overrides approval above)</param>
        /// <param name="sortData">sort parameter</param>
        /// <returns>Returns a Collection of Users (as recommended by FxCop)</returns>
        /// 
        [DataObjectMethod(DataObjectMethodType.Select, false)]
        static public List<MembershipUserWrapper> GetMembers(bool returnAllApprovedUsers, bool returnAllNotApprovedUsers,
            string usernameToFind, string sortData)
        {

            List<MembershipUserWrapper> memberList = new List<MembershipUserWrapper>();

            // See if we are looking for just one user
            if (usernameToFind != null)
            {
                MembershipUser mu = Membership.GetUser(usernameToFind);
                if (mu != null)
                {
                    MembershipUserWrapper md = new MembershipUserWrapper(mu);
                    memberList.Add(md);
                }
            }
            else
            {
                MembershipUserCollection muc = Membership.GetAllUsers();
                foreach (MembershipUser mu in muc)
                {
                    if ((returnAllApprovedUsers == true && mu.IsApproved == true) ||
                         (returnAllNotApprovedUsers == true && mu.IsApproved == false))
                    {
                        MembershipUserWrapper md = new MembershipUserWrapper(mu);
                        memberList.Add(md);
                    }
                }

  */
