/*
Copyright  2006, Bilal Haidar
All rights reserved.
http://www.bhaidar.net

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

- Redistributions of source code _must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Neither Bilal Haidar, nor the names of its
contributors may be used to endorse or promote products
derived from this software without specific prior written 
permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using System.ComponentModel;

namespace ExtendedMemberShip
{
    /// <summary>
    /// Summary description for MemberShipUser
    /// This class inherits from the MembershipUser class
    /// using the syntax public class ClassName (...) : base(...) allows for calling
    /// the constructor of the base class, which in this case is MembershipUser
    /// </summary>
    public class MemberShipUser : MembershipUser
    {
        #region Constructors
        /// <summary>
        /// This is the default empty constructor
        /// </summary>
        public MemberShipUser()
            : base()
        {
        }

        /// <summary>
        /// This constructor is used to create a MemberShipUser out of a MembershipUser object with 
        /// the additional MemberShipUser fields that belong to this class (MemberShipUser)
        /// </summary>
        /// <param name="_mu">MembershipUser object</param>
        /// <param name="FirstName">First name of the user</param>
        /// <param name="LastName">Last name of the user</param>
        /// <param name="DateOfBirth">Date of birth of the user</param>
        public MemberShipUser(MembershipUser _mu, string FirstName, string LastName, DateTime DateOfBirth)
            : base(_mu.ProviderName, _mu.UserName, _mu.ProviderUserKey, 
            _mu.Email, _mu.PasswordQuestion, _mu.Comment, 
            _mu.IsApproved, _mu.IsLockedOut, _mu.CreationDate, 
            _mu.LastLoginDate, _mu.LastActivityDate, _mu.LastPasswordChangedDate, 
            _mu.LastLockoutDate)
        {
            // Assign local fields
            this._FirstName = FirstName;
            this._LastName = LastName;
            this._DateOfBirth = DateOfBirth;
        }

        /// <summary>
        /// This constructor is used to create a MemberShipUser out of the individual MembershipUser object parameters
        /// with the additional MemberShipUser fields that belong to this class (MemberShipUser)
        /// </summary>
        /// <param name="_mu">MembershipUser object</param>
        /// <param name="FirstName">First name of the user</param>
        /// <param name="LastName">Last name of the user</param>
        /// <param name="DateOfBirth">Date of birth of the user</param>
        public MemberShipUser(string ProviderName,  string UserName, object ProviderUserKey, string Email, string PasswordQuestion,
            string Comment, bool IsApproved, bool IsLockedOut, DateTime CreationDate, DateTime LastLoginDate, DateTime LastActivityDate, 
            DateTime LastPasswordChangedDate, DateTime LastLockoutDate, string FirstName, string LastName, DateTime DateOfBirth)
            : base(ProviderName, UserName, ProviderUserKey, Email, PasswordQuestion, Comment, IsApproved, IsLockedOut, 
            CreationDate, LastLoginDate, LastActivityDate, LastPasswordChangedDate, LastLockoutDate)
        {
            // Assign local fields
            this._FirstName = FirstName;
            this._LastName = LastName;
            this._DateOfBirth = DateOfBirth;
        }
        #endregion

        #region Private fields
        private string _FirstName;
        private string _LastName;
        private DateTime _DateOfBirth;
        #endregion

        #region Public properties
        /// <summary>
        /// This local property overrides the bas property ProviderUserKey.
        /// This property will serve as the primary key of the MemberShipUser
        /// specially useful when working with data controls.
        /// </summary>
        [DataObjectFieldAttribute(true,false,false)]
        public override object ProviderUserKey
        {
            get { return base.ProviderUserKey; }
        }
        /// <summary>
        /// A custom property added to the MemberShipUser class
        /// that represents the First Name of the user
        /// </summary>
        [DataObjectFieldAttribute(false,false,true)]
        public string FirstName
        {
            get
            { 
                return _FirstName;
            }
        }
        /// <summary>
        /// A custom property added to the MemberShipUser class
        /// that represents the Last Name of the user
        /// </summary>
        [DataObjectFieldAttribute(false,false,true)]
        public string LastName
        {
            get
            {
                return _LastName;
            }
        }

        /// <summary>
        /// A custom property added to the MemberShipUser class
        /// that represents the date of birth of the user
        /// </summary>
        [DataObjectFieldAttribute(false,false,true)]
        public DateTime DateOfBirth
        {
            get
            {
                return _DateOfBirth;
            }
            set
            {
                this._DateOfBirth = value;
            }
        }
        #endregion
    }
}