using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Configuration.Provider;
using System.Web;
using System.Web.Security;

namespace ExtendedMemberShip
{
	public class SqlMemberShipProvider : MemberShipProvider
	{
        #region Implement all the methods in the MemberShipProvider
        /// <summary>
        /// This method adds a new reocrd to our custom table
        /// </summary>
        /// <param name="msu"></param>
        /// <returns>MemberShipUser instance</returns>
        internal override bool CreateUser(UserInfo userInfo)
        {
            // Do some validation on the parameters
            if ((userInfo == null) || (userInfo.UserId == null))
                // Failed, no user details creation
                return false;

            if (string.IsNullOrEmpty(userInfo.FirstName) || (string.IsNullOrEmpty(userInfo.LastName)) || 
                (userInfo.DateOfBirth < DateTime.MinValue))
                // Failed, no user details creation
                return false;

                // Sql Connection
                SqlConnection con = null;
                try
                {
                    con = new SqlConnection(this.connectionString);

                    SqlCommand command1 = new SqlCommand("dbo.aspnet_ExtendedMembership_CreateUser", con);
                    command1.CommandType = CommandType.StoredProcedure;
                    command1.Parameters.Add(this.CreateInputParam("@UserId", SqlDbType.UniqueIdentifier, userInfo.UserId));
                    command1.Parameters.Add(this.CreateInputParam("@FirstName", SqlDbType.VarChar, userInfo.FirstName));
                    command1.Parameters.Add(this.CreateInputParam("@LastName", SqlDbType.VarChar, userInfo.LastName));
                    command1.Parameters.Add(this.CreateInputParam("@DateOfBirth", SqlDbType.DateTime, userInfo.DateOfBirth));
                    SqlParameter parameter1 = new SqlParameter("@ReturnValue", SqlDbType.Int);
                    parameter1.Direction = ParameterDirection.ReturnValue;
                    command1.Parameters.Add(parameter1);

                    // Execute the command
                    con.Open();
                    command1.ExecuteNonQuery();
                    int result = (parameter1.Value != null) ? ((int)parameter1.Value) : -1;

                    return (result == 0);
                }
                catch
                {
                    throw new ApplicationException("Exception in adding user's details");
                }
                finally
                {
                    if (con != null)
                    {
                        con.Close();
                        con = null;
                    }
                }
        }

        /// <summary>
        /// This method deletes a user from our custom table
        /// </summary>
        /// <param name="UserId"></param>
        internal override bool Delete(object UserId)
        {
            // Do some validation on the parameters
            if (UserId == null)
                throw new ApplicationException("UserId cannot be null");

            // Sql Connection
            SqlConnection con = null;
            try
            {
                con = new SqlConnection(this.connectionString);

                SqlCommand command1 = new SqlCommand("dbo.aspnet_ExtendedMembership_DeleteUser", con);
                command1.CommandType = CommandType.StoredProcedure;
                command1.Parameters.Add(this.CreateInputParam("@UserId", SqlDbType.UniqueIdentifier, UserId));
                SqlParameter parameter1 = new SqlParameter("@NumTablesDeletedFrom", SqlDbType.Int);
                parameter1.Direction = ParameterDirection.Output;
                command1.Parameters.Add(parameter1);

                // Execute the command
                con.Open();
                command1.ExecuteNonQuery();
                int num1 = (parameter1.Value != null) ? ((int)parameter1.Value) : -1;
                return (num1 > 0);
            }
            catch
            {
                throw new ApplicationException("Exception in deleting a user details");
            }
            finally
            {
                if (con != null)
                {
                    con.Close();
                    con = null;
                }
            }
        }

        /// <summary>
        /// This method is used to update the DateOfBirth of the user
        /// Although a person date of birth never changes, however
        /// I inlcuded this method to have a full implementation of
        /// all the basic methods needed.
        /// </summary>
        /// <param name="DateOfBith"></param>
        internal override void UpdateUser(object UserId, DateTime DateOfBith)
        {
            // Do some validation on the parameters
            if (UserId == null)
                throw new ApplicationException("UserId cannot be null");

            if (DateOfBith <= DateTime.MinValue)
                throw new ApplicationException("Date of birth value is invalid");

            // Sql Connection
            SqlConnection con = null;
            try
            {
                con = new SqlConnection(this.connectionString);

                SqlCommand command1 = new SqlCommand("dbo.aspnet_ExtendedMembership_UpdateUser", con);
                command1.CommandType = CommandType.StoredProcedure;
                command1.Parameters.Add(this.CreateInputParam("@UserId", SqlDbType.UniqueIdentifier, UserId));
                command1.Parameters.Add(this.CreateInputParam("@DateOfBirth", SqlDbType.DateTime, DateOfBith));
                SqlParameter parameter1 = new SqlParameter("@ReturnValue", SqlDbType.Int);
                parameter1.Direction = ParameterDirection.ReturnValue;
                command1.Parameters.Add(parameter1);

                // Execute the command
                con.Open();
                command1.ExecuteNonQuery();
                int result = (parameter1.Value != null) ? ((int)parameter1.Value) : -1;
                if (result != 0)
                {
                    // Something wrong went with the execution
                    // Update didn't work properly
                }
            }
            catch
            {
                throw new ApplicationException("Exception in updating a user details");
            }
            finally
            {
                if (con != null)
                {
                    con.Close();
                    con = null;
                }
            }
        }

        /// <summary>
        /// This methods will retreive user's detail from our custom table
        /// </summary>
        /// <param name="UserId"></param>
        /// <returns></returns>
        internal override UserInfo GetUser(object UserId)
        {
            // Sql Connection
            SqlConnection con = null;
            // Define Data reader
            SqlDataReader reader1 = null;
            try
            {
                con = new SqlConnection(this.connectionString);


                SqlCommand command1 = new SqlCommand("dbo.aspnet_ExtendedMembership_GetUserByUserId", con);
                command1.CommandType = CommandType.StoredProcedure;
                command1.Parameters.Add(this.CreateInputParam("@UserId", SqlDbType.UniqueIdentifier, UserId));
                SqlParameter parameter1 = new SqlParameter("@ReturnValue", SqlDbType.Int);
                parameter1.Direction = ParameterDirection.ReturnValue;
                command1.Parameters.Add(parameter1);

                // Execute the Reader
                con.Open();
                reader1 = command1.ExecuteReader(CommandBehavior.CloseConnection);
                if (reader1.Read())
                {
                    string text1 = reader1.GetString(0);
                    string text2 = reader1.GetString(1);
                    DateTime time1 = reader1.GetDateTime(2);
                    return new UserInfo(UserId,text1,text2,time1);
                }
              return null;
            }
            catch
            {
                throw new ApplicationException("Exception in GetUser");
            }
            finally
            {
                if (reader1 != null)
                {
                    reader1.Close();
                    reader1 = null;
                }
                if (con != null)
                {
                    con.Close();
                    con = null;
                }
            }

            return null;
        }

        /// <summary>
        /// This method retreives a list of all user's details in our custom table
        /// </summary>
        /// <returns></returns>
        internal override List<UserInfo> GetAllUsers()
        {
             List<UserInfo> collection1 = new List<UserInfo>();
            
            // Sql Connection
            SqlConnection con = new SqlConnection(this.connectionString);
            // Define the data reader
            SqlDataReader reader1 = null;
            try
            {
                con = new SqlConnection(this.connectionString);
                
                SqlCommand command1 = new SqlCommand("dbo.aspnet_ExtendedMembership_GetAllUsers", con);
                command1.CommandType = CommandType.StoredProcedure;

                // Open connection
                con.Open();
                reader1 = command1.ExecuteReader(CommandBehavior.CloseConnection);
                while (reader1.Read())
                {
                    Guid guid1 = reader1.GetGuid(0);
                    string text1 = reader1.GetString(1);
                    string text2 = reader1.GetString(2);
                    DateTime time1 = reader1.GetDateTime(3);
                    
                    // Add to collection
                    collection1.Add(new UserInfo(guid1,text1,text2,time1));
                }
                
                return collection1;
            }
            catch
            {
                throw new ApplicationException("Exception in GetAllUsers");
            }
            finally
            {
                if (reader1 != null)
                {
                    reader1.Close();
                    reader1 = null;
                }
                if (con != null)
                {
                    con.Close();
                    con = null;
                }
            }
        }

        /// <summary>
        /// This method retreives a list of users based on the UserIds passed to it
        /// as a paramter
        /// </summary>
        /// <param name="UserIds"></param>
        /// <returns></returns>
        internal override List<UserInfo> GetUsersByUserIds(string UserIds)
        {
            // Do some validation on the parameters
            if (string.IsNullOrEmpty(UserIds))
                return null;

            List<UserInfo> collection1 = new List<UserInfo>();
            
            // Sql Connection
            SqlConnection con = new SqlConnection(this.connectionString);
            // Define the data reader
            SqlDataReader reader1 = null;
            try
            {
                con = new SqlConnection(this.connectionString);

                SqlCommand command1 = new SqlCommand("dbo.aspnet_ExtendedMembership_GetUsersByUserIds", con);
                command1.CommandType = CommandType.StoredProcedure;
                command1.Parameters.Add(this.CreateInputParam("@ListOfUserIds", SqlDbType.VarChar, UserIds));

                // Open connection
                con.Open();
                reader1 = command1.ExecuteReader(CommandBehavior.CloseConnection);
                while (reader1.Read())
                {
                    Guid guid1 = reader1.GetGuid(0);
                    string text1 = reader1.GetString(1);
                    string text2 = reader1.GetString(2);
                    DateTime time1 = reader1.GetDateTime(3);
                    
                    // Add to collection
                    collection1.Add(new UserInfo(guid1, text1, text2, time1));
                }
                
                return collection1;
            }
            catch
            {
                //throw new ApplicationException("Exception in GetUsersByUserIds");
            }
            finally
            {
                if (reader1 != null)
                {
                    reader1.Close();
                    reader1 = null;
                }
                if (con != null)
                {
                    con.Close();
                    con = null;
                }
            }

            return null;
        }
        #endregion  

        #region Helper Methods
        private SqlParameter CreateInputParam(string paramName, SqlDbType dbType, object objValue)
        {
            SqlParameter parameter1 = new SqlParameter(paramName, dbType);
            if (objValue == null)
            {
                parameter1.IsNullable = true;
                parameter1.Value = DBNull.Value;
                return parameter1;
            }
            parameter1.Value = objValue;
            return parameter1;
        }
        #endregion

        #region Provider Section
        private string connectionString;
		public override void Initialize(string name, System.Collections.Specialized.NameValueCollection config)
		{
			if ((config == null) || (config.Count == 0))
				throw new ArgumentNullException("You must supply a valid configuration dictionary.");

			if (string.IsNullOrEmpty(config["description"]))
			{
				config.Remove("description");
				config.Add("description", "Put a localized description here.");
			}

			// Let ProviderBase perform the basic initialization
			base.Initialize(name, config);

			// Perform feature-specific provider initialization here
			// Get the connection string
			string connectionStringName = config["connectionStringName"];
			if (String.IsNullOrEmpty(connectionStringName))
				throw new ProviderException("You must specify a connectionStringName attribute.");

			ConnectionStringsSection cs = (ConnectionStringsSection)ConfigurationManager.GetSection("connectionStrings");

			if (cs == null)
				throw new ProviderException("An error occurred retrieving the connection strings section.");

			if (cs.ConnectionStrings[connectionStringName] == null)
				throw new ProviderException("The connection string could not be found in the connection strings section.");
			else
				connectionString = cs.ConnectionStrings[connectionStringName].ConnectionString;

			if (String.IsNullOrEmpty(connectionString))
				throw new ProviderException("The connection string is invalid.");

			config.Remove("connectionStringName");

			// Check to see if unexpected attributes were set in configuration
			if (config.Count > 0)
			{
				string extraAttribute = config.GetKey(0);

				if (!String.IsNullOrEmpty(extraAttribute))
					throw new ProviderException("The following unrecognized attribute was found in " + Name + "'s configuration: '" + extraAttribute + "'");
				else
					throw new ProviderException("An unrecognized attribute was found in the provider's configuration.");
			}
        }
        #endregion
    }
}